import java.util.Map;
import java.util.Vector;
import java.util.Hashtable;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;
import javax.naming.NamingEnumeration;
import javax.naming.directory.*;
import javax.naming.NamingException;

/**
 *
 */
public class LDAPManager
{
    protected String factory = "com.sun.jndi.ldap.LdapCtxFactory";
    protected String serverName = "virtuas-demo";
    protected int port = 389;
    protected String securityAuthentication = "simple";
    protected String dirManager = "cn=Directory Manager";
    protected String dirManagerPw = "directory";
    protected String searchBase = null;
    protected int searchScope = SearchControls.SUBTREE_SCOPE;

    public LDAPManager()
    {
    }

    /**
     * Returns all of the necessary environment information required to
     * create a Context and bind to an LDAP server.
     */
    protected Hashtable getEnvironment()
    {
        Hashtable env = new Hashtable();

        // generate LDAP host URL
        StringBuffer url = new StringBuffer("ldap://");
        url.append(serverName).append(":").append(port);

        // set Context Factory class
        env.put(javax.naming.Context.INITIAL_CONTEXT_FACTORY, factory);
        // set host URL
        env.put(javax.naming.Context.PROVIDER_URL, url.toString());

        // Only set authentication information if there is a username
        // present
        if ((dirManager != null) && (dirManager.trim().length() > 0))
        {
            env.put(javax.naming.Context.SECURITY_AUTHENTICATION,
                    securityAuthentication);
            env.put(javax.naming.Context.SECURITY_PRINCIPAL, dirManager);
            env.put(javax.naming.Context.SECURITY_CREDENTIALS, dirManagerPw);
        }

        return env;
    }

    /**
     * Creates a new Directory Context based on the local settings.
     */
    protected DirContext getInitialContext(Hashtable env) throws NamingException
    {
        return new InitialDirContext(env);
    }

    /**
     * Searches the ldap server using the search filter that is passed to the
     * method.
     */
    public Vector search(String[] returnAttributes, String searchFilter)
        throws NamingException
    {
        Vector ldapObjects = new Vector();
        SearchControls ctls = new SearchControls();
        ctls.setReturningAttributes(returnAttributes);
        ctls.setSearchScope(searchScope);
        NamingEnumeration results = null;

        // bind to the LDAP server.
        DirContext ctx = getInitialContext(getEnvironment());

        // perform the search on the LDAP server.
        results = ctx.search(searchBase, searchFilter, ctls);
        while ((results != null) && (results.hasMore()))
        {
            // get the next set of results from the search.
            SearchResult sr = (SearchResult)results.next();
            // get all of the attributes for this object.
            Attributes attributes = sr.getAttributes();
            NamingEnumeration attEnum = attributes.getAll();
            // create an LDAPObject to hold the results.
            LDAPObject tmpObj = new LDAPObject();

            // set the Distinguished Name for the object. Append
            // the relative dn and the searcb base together.
            tmpObj.setDn(sr.getName() + "," + searchBase);
            // iterate through the attributes setting them in the LDAPObject
            while (attEnum.hasMore())
            {
                // get the next attribute
                Attribute att = (Attribute)attEnum.next();
                // set the attribute in the LDAPObject
                tmpObj.setAttribute(att.getID(), att.get());
            }
            // add the object to the list of return objects
            ldapObjects.add(tmpObj);
        }

        return ldapObjects;
    }

    /**
     * Updates the given object in the LDAP server.
     */
    public void modify(String dn, HashMap attributes, int modOp)
        throws NamingException
    {
        // perform the modification on all of the given attributes
        // based on the modOp that was passed in
        DirContext ctx = getInitialContext(getEnvironment());
        ctx.modifyAttributes(dn, modOp, parseAttributes(attributes));
    }

    /**
     * Adds a new object to the LDAP server with the give attributes.
     */
    public void add(String dn, HashMap attributes, String[] objectClass)
        throws NamingException
    {
        // parse the attributes that were passed in
        Attributes atts = parseAttributes(attributes);

        // add objectClass attribute to the list of attributes
        Attribute oc = new BasicAttribute("objectClass");
        for (int i = 0; i < objectClass.length; i++)
        {
            oc.add(objectClass[i]);
        }
        atts.put(oc);

        // perform the addition to the LDAP server
        DirContext ctx = getInitialContext(getEnvironment());
        ctx.bind(dn, null, atts);
    }

    /**
     * Remove the specified object from the LDAP server.
     */
    public void remove(String dn) throws NamingException
    {
        // perform the deletion from the LDAP server based on the objects DN
        DirContext ctx = getInitialContext(getEnvironment());
        ctx.destroySubcontext(dn);
    }

    public Attributes parseAttributes(HashMap attributes)
    {
        if (attributes == null)
        {
            return null;
        }

        Attributes newAtts = new BasicAttributes();

        // get an iterator for all of the keys in the HashMap
        Iterator attIter = attributes.keySet().iterator();
        while (attIter.hasNext())
        {
            // get the value for the key.
            String attName = (String)attIter.next();
            Object value = attributes.get(attName);

            // create a new Attribute
            Attribute attribute = new BasicAttribute(attName, value);

            // add the attribute to the list of attributes
            newAtts.put(attribute);
        }

        return newAtts;
    }

    public void setFactory(String factory)
    {
        this.factory = factory;
    }

    public String getServerName()
    {
        return serverName;
    }

    public void setServerName(String serverName)
    {
        this.serverName = serverName;
    }

    public int getPort()
    {
        return port;
    }

    public void setPort(int port)
    {
        this.port = port;
    }

    public String getSearchBase()
    {
        return searchBase;
    }

    public void setSearchBase(String sb)
    {
        searchBase = sb;
    }

    public String getSecurityAuthentication()
    {
        return securityAuthentication;
    }

    public void setSecurityAuthentication(String sa)
    {
        this.securityAuthentication = sa;
    }

    public String getDirManager()
    {
        return dirManager;
    }

    public void setDirManager(String dirManager)
    {
        this.dirManager = dirManager;
    }

    public String getDirManagerPw()
    {
        return dirManagerPw;
    }

    public void setDirManagerPw(String dirManagerPw)
    {
        this.dirManagerPw = dirManagerPw;
    }

    public int getSearchScope()
    {
        return searchScope;
    }

    public void setSearchScope(int sc)
    {
        searchScope = sc;
    }
}
